/*
 * Copyright (C) 2011 Technische Universitaet Muenchen
 * This file is part of the Sierpi project. For conditions of distribution and
 * use, please see the copyright notice in the file 'copyright.txt' at the root
 * directory of this package and the copyright notice at http://www5.in.tum.de/sierpi
 */
/*
 * CGenericTreeNode_serial.hpp
 *
 *  Created on: Sep 25, 2011
 *      Author: Martin Schreiber (martin.schreiber@in.tum.de)
 */


#ifndef CGENERICTREENODE_SERIAL_HPP_
#define CGENERICTREENODE_SERIAL_HPP_

/**
 * !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 * !!! THIS FILE IS INCLUDED DIRECTLY TO THE CGenericTreeNode Class !!!
 * !!! Therefore the following variables are members of the class   !!!
 * !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 */

inline void specializedConstructorMethod()
{
}


/****************************************************************************
 * GENERIC TREE NODE (NO REDUCE)
 */
template <
	typename CLambdaFun
>
class CTraversalTask_GenericTreeNode
{
	CGenericTreeNode_ *this_node;
	CLambdaFun lambda;

public:
	CTraversalTask_GenericTreeNode(
			CGenericTreeNode_ *p_this_node,
			CLambdaFun p_lambda
	)	:
		this_node(p_this_node),
		lambda(p_lambda)
	{
	}

	void* execute()
	{
		if (this_node->isLeaf())
		{
			lambda(this_node);
			return 0;
		}

		CTraversalTask_GenericTreeNode<CLambdaFun> first(this_node->first_child_node, lambda);
		CTraversalTask_GenericTreeNode<CLambdaFun> second(this_node->second_child_node, lambda);

		if (this_node->first_child_node)
			first.execute();

		if (this_node->second_child_node)
			second.execute();

		return 0;
	}
};


template <bool parallelProcessing, typename CLambdaFun>
void traverse_GenericTreeNode(
		CLambdaFun p_lambda
)
{
	CTraversalTask_GenericTreeNode<CLambdaFun>(this, p_lambda).execute();
}


template <typename CLambdaFun>
void traverse_GenericTreeNode_Parallel(
		CLambdaFun p_lambda
)
{
	CTraversalTask_GenericTreeNode<CLambdaFun>(this, p_lambda).execute();
}


template <typename CLambdaFun>
void traverse_GenericTreeNode_Serial(
		CLambdaFun p_lambda
)
{
	CTraversalTask_GenericTreeNode<CLambdaFun>(this, p_lambda).execute();
}



/****************************************************************************
 * GENERIC TREE NODE (NO REDUCE) MID AND LEAF NODES
 */
template <
	typename CLambdaFun1,
	typename CLambdaFun2
>
class CTraversalTask_GenericTreeNode_MidAndLeafNodes
{
	CGenericTreeNode_ *this_node;
	CLambdaFun1 lambda_leaves;
	CLambdaFun2 lambda_midnodes;

public:
	CTraversalTask_GenericTreeNode_MidAndLeafNodes(
			CGenericTreeNode_ *p_this_node,
			CLambdaFun1 p_lambda_leaves,
			CLambdaFun2 p_lambda_midnodes
	)	:
		this_node(p_this_node),
		lambda_leaves(p_lambda_leaves),
		lambda_midnodes(p_lambda_midnodes)
	{
	}

	void* execute()
	{
		if (this_node->isLeaf())
		{
			lambda_leaves(this_node);
			return 0;
		}

		CTraversalTask_GenericTreeNode_MidAndLeafNodes<CLambdaFun1, CLambdaFun2> first(this_node->first_child_node, lambda_leaves, lambda_midnodes);
		CTraversalTask_GenericTreeNode_MidAndLeafNodes<CLambdaFun1, CLambdaFun2> second(this_node->second_child_node, lambda_leaves, lambda_midnodes);

		if (this_node->first_child_node)
			first.execute();

		if (this_node->second_child_node)
			second.execute();

		if (this_node)
			lambda_midnodes(this_node);

		return 0;
	}
};


template <bool parallelProcessing, typename CLambdaFun1, typename CLambdaFun2>
void traverse_GenericTreeNode_MidAndLeafNodes(
		CLambdaFun1 p_lambda_leaves,
		CLambdaFun2 p_lambda_midnodes
)
{
	CTraversalTask_GenericTreeNode_MidAndLeafNodes<CLambdaFun1, CLambdaFun2>(this, p_lambda_leaves, p_lambda_midnodes).execute();
}


template <typename CLambdaFun1, typename CLambdaFun2>
void traverse_GenericTreeNode_MidAndLeafNodes_Serial(
		CLambdaFun1 p_lambda_leaves,
		CLambdaFun2 p_lambda_midnodes
)
{
	CTraversalTask_GenericTreeNode_MidAndLeafNodes<CLambdaFun1, CLambdaFun2>(this, p_lambda_leaves, p_lambda_midnodes).execute();
}



template <typename CLambdaFun1, typename CLambdaFun2>
void traverse_GenericTreeNode_MidAndLeafNodes_Parallel(
		CLambdaFun1 p_lambda_leaves,
		CLambdaFun2 p_lambda_midnodes
)
{
	CTraversalTask_GenericTreeNode_MidAndLeafNodes<CLambdaFun1, CLambdaFun2>(this, p_lambda_leaves, p_lambda_midnodes).execute();
}




/****************************************************************************
 * GENERIC TREE NODE AND DEPTH FOR ALL MID AND LEAF NODES + DEPTH
 */

/*
 * parallel
 */
template <
	typename CLambdaFun1,
	typename CLambdaFun2
>
class CTraversalTask_GenericTreeNode_MidAndLeafNodes_Depth_Parallel
{
public:
	static void traverse(
			CGenericTreeNode_ *this_node,
			CLambdaFun1 lambda_leaves,
			CLambdaFun2 lambda_midnodes,
			int p_genericTreeDepth
	)
	{
		if (this_node->isLeaf())
		{
			lambda_leaves(this_node, p_genericTreeDepth);
			return;
		}

		p_genericTreeDepth++;

		if (this_node->first_child_node)
		{
			traverse(this_node->first_child_node, lambda_leaves, lambda_midnodes, p_genericTreeDepth);
		}

		if (this_node->second_child_node)
		{
			traverse(this_node->second_child_node, lambda_leaves, lambda_midnodes, p_genericTreeDepth);
		}

		lambda_midnodes(this_node, p_genericTreeDepth-1);
	}
};


template <typename CLambdaFun1, typename CLambdaFun2>
void traverse_GenericTreeNode_MidAndLeafNodes_Depth_Parallel(
		CLambdaFun1 p_lambda_leaves,
		CLambdaFun2 p_lambda_midnodes,
		int p_genericTreeDepth
)
{
	CTraversalTask_GenericTreeNode_MidAndLeafNodes_Depth_Parallel<CLambdaFun1, CLambdaFun2>::traverse(this, p_lambda_leaves, p_lambda_midnodes, 0);
}


/*
 * serial
 */
template <
	typename CLambdaFun1,
	typename CLambdaFun2
>
class CTraversalTask_GenericTreeNode_MidAndLeafNodes_Depth_Serial
{
public:
	static void traverse(
			CGenericTreeNode_ *this_node,
			CLambdaFun1 lambda_leaves,
			CLambdaFun2 lambda_midnodes,
			int p_genericTreeDepth
	)
	{
		if (this_node->isLeaf())
		{
			lambda_leaves(this_node, p_genericTreeDepth);
			return;
		}

		p_genericTreeDepth++;
		if (this_node->first_child_node)
		{
			traverse(this_node->first_child_node, lambda_leaves, lambda_midnodes, p_genericTreeDepth);
		}

		if (this_node->second_child_node)
		{
			traverse(this_node->second_child_node, lambda_leaves, lambda_midnodes, p_genericTreeDepth);
		}

		lambda_midnodes(this_node, p_genericTreeDepth-1);
	}
};



template <typename CLambdaFun1, typename CLambdaFun2>
void traverse_GenericTreeNode_MidAndLeafNodes_Depth_Serial(
		CLambdaFun1 p_lambda_leaves,
		CLambdaFun2 p_lambda_midnodes
)
{
	CTraversalTask_GenericTreeNode_MidAndLeafNodes_Depth_Serial<CLambdaFun1, CLambdaFun2>::traverse(this, p_lambda_leaves, p_lambda_midnodes, 0);
}



/*
 * parallel/serial
 */
template <bool parallelProcessing, typename CLambdaFun1, typename CLambdaFun2>
void traverse_GenericTreeNode_MidAndLeafNodes_Depth(
		CLambdaFun1 p_lambda_leaves,
		CLambdaFun2 p_lambda_midnodes
)
{
	CTraversalTask_GenericTreeNode_MidAndLeafNodes_Depth_Serial<CLambdaFun1, CLambdaFun2>::traverse(this, p_lambda_leaves, p_lambda_midnodes, 0);
}



/****************************************************************************
 * GENERIC TREE NODE (WITH REDUCE)
 ****************************************************************************/
template <
	typename CLambdaFun,
	typename TReduceValue
>
class CTraversalTask_GenericTreeNode_Reduce
{
	CGenericTreeNode_ *this_node;
	CLambdaFun lambda;
	void (*reduceOperator)(const TReduceValue &i_a, const TReduceValue &i_b, TReduceValue *o_reduceOutput);

public:
	TReduceValue reduceValue;

	inline CTraversalTask_GenericTreeNode_Reduce(
			CGenericTreeNode_ *p_this_node,
			CLambdaFun p_lambda,
			void (*p_reduceOperator)(const TReduceValue &i_a, const TReduceValue &i_b, TReduceValue *o_reduceOutput)
	)	:
		this_node(p_this_node),
		lambda(p_lambda),
		reduceOperator(p_reduceOperator)
	{
	}

	/**
	 * TASK
	 */
	void* execute()
	{
		/**
		 * LEAF COMPUTATION
		 */
		if (this_node->isLeaf())
		{
			lambda(this_node, &reduceValue);
			return 0;
		}

		/**
		 * PARALLEL TRAVERSALS
		 */

		/**
		 * first and second child node exists
		 */
		if (this_node->first_child_node && this_node->second_child_node)
		{
			CTraversalTask_GenericTreeNode_Reduce<CLambdaFun, TReduceValue> first(this_node->first_child_node, lambda, reduceOperator);
			first.execute();

			CTraversalTask_GenericTreeNode_Reduce<CLambdaFun, TReduceValue> second(this_node->second_child_node, lambda, reduceOperator);
			second.execute();

			reduceOperator(first.reduceValue, second.reduceValue, &reduceValue);

			return nullptr;
		}

		if (this_node->first_child_node)
		{
			CTraversalTask_GenericTreeNode_Reduce<CLambdaFun, TReduceValue> first(this_node->first_child_node, lambda, reduceOperator);
			first.execute();
			reduceValue = first.reduceValue;

			return nullptr;
		}

		assert(this_node->second_child_node != nullptr);

		CTraversalTask_GenericTreeNode_Reduce<CLambdaFun, TReduceValue> second(this_node->second_child_node, lambda, reduceOperator);
		second.execute();
		reduceValue = second.reduceValue;

		return nullptr;
	}
};

template <bool parallelProcessing, typename CLambdaFun, typename TReduceValue>
void traverse_GenericTreeNode_Reduce(
		CLambdaFun lambda,
		void (*p_reduceOperator)(const TReduceValue &i_a, const TReduceValue &i_b, TReduceValue *o_reduceOutput),
		TReduceValue *o_reduceOutput
)
{
	// start root task
	CTraversalTask_GenericTreeNode_Reduce<CLambdaFun, TReduceValue> root(this, lambda, p_reduceOperator);
	root.execute();

	*o_reduceOutput = root.reduceValue;
	return;
}

template <typename CLambdaFun, typename TReduceValue>
void traverse_GenericTreeNode_Reduce_Serial(
		CLambdaFun i_lambdaFun,
		void (*i_reduceOperator)(const TReduceValue &i_a, const TReduceValue &i_b, TReduceValue *o_reduceOutput),
		TReduceValue *o_reduceOutput
)
{
	traverse_GenericTreeNode_Reduce<false, CLambdaFun, TReduceValue>(i_lambdaFun, i_reduceOperator, o_reduceOutput);
}

template <typename CLambdaFun, typename TReduceValue>
void traverse_GenericTreeNode_Reduce_Parallel(
		CLambdaFun i_lambdaFun,
		void (*i_reduceOperator)(const TReduceValue &i_a, const TReduceValue &i_b, TReduceValue *o_reduceOutput),
		TReduceValue *o_reduceOutput
)
{
	traverse_GenericTreeNode_Reduce<true, CLambdaFun, TReduceValue>(i_lambdaFun, i_reduceOperator, o_reduceOutput);
}





/****************************************************************************
 * GENERIC TREE NODE MidAndLeafNodes (WITH REDUCE)
 ****************************************************************************/

template <
	typename CLambdaFun1,
	typename CLambdaFun2,
	typename TReduceValue
>
class CTraversalTask_GenericTreeNode_MidAndLeafNodes_Reduce
{
	CGenericTreeNode_ *this_node;
	CLambdaFun1 lambda_leaves;
	CLambdaFun2 lambda_midnodes;
	void (*reduceOperator)(const TReduceValue &i_a, const TReduceValue &i_b, TReduceValue *o_reduceOutput);

public:
	TReduceValue reduceValue;

	inline CTraversalTask_GenericTreeNode_MidAndLeafNodes_Reduce(
			CGenericTreeNode_ *p_this_node,
			CLambdaFun1 i_lambda_leaves,
			CLambdaFun2 i_lambda_midnodes,
			void (*p_reduceOperator)(const TReduceValue &i_a, const TReduceValue &i_b, TReduceValue *o_reduceOutput)
	)	:
		this_node(p_this_node),
		lambda_leaves(i_lambda_leaves),
		lambda_midnodes(i_lambda_midnodes),
		reduceOperator(p_reduceOperator)
	{
	}

	/**
	 * TASK
	 */
	void execute()
	{
		/**
		 * LEAF COMPUTATION
		 */
		if (this_node->isLeaf())
		{
			lambda_leaves(this_node, &reduceValue);
			return;
		}

		/**
		 * PARALLEL TRAVERSALS
		 */

		/**
		 * first and second child node exists
		 */
		if (this_node->first_child_node && this_node->second_child_node)
		{
			CTraversalTask_GenericTreeNode_MidAndLeafNodes_Reduce<CLambdaFun1, CLambdaFun2, TReduceValue> first(this_node->first_child_node, lambda_leaves, lambda_midnodes, reduceOperator);
			first.execute();

			CTraversalTask_GenericTreeNode_MidAndLeafNodes_Reduce<CLambdaFun1, CLambdaFun2, TReduceValue> second(this_node->second_child_node, lambda_leaves, lambda_midnodes, reduceOperator);
			second.execute();

			reduceOperator(first.reduceValue, second.reduceValue, &reduceValue);

			lambda_midnodes(this_node, &reduceValue);

			return;
		}

		if (this_node->first_child_node)
		{
			CTraversalTask_GenericTreeNode_MidAndLeafNodes_Reduce<CLambdaFun1, CLambdaFun2, TReduceValue> first(this_node->first_child_node, lambda_leaves, lambda_midnodes, reduceOperator);
			first.execute();
			reduceValue = first.reduceValue;

			lambda_midnodes(this_node, &reduceValue);

			return;
		}

		assert(this_node->second_child_node != nullptr);

		CTraversalTask_GenericTreeNode_MidAndLeafNodes_Reduce<CLambdaFun1, CLambdaFun2, TReduceValue> second(this_node->second_child_node, lambda_leaves, lambda_midnodes, reduceOperator);
		second.execute();
		reduceValue = second.reduceValue;

		lambda_midnodes(this_node, &reduceValue);

		return;
	}
};



template <
	typename CLambdaFun1,
	typename CLambdaFun2,
	typename TReduceValue
>
void traverse_GenericTreeNode_MidAndLeafNodes_Reduce_Serial(
		CLambdaFun1 i_lambda_leaves,
		CLambdaFun2 i_lambda_midnodes,
		void (*i_reduceOperator)(const TReduceValue &i_a, const TReduceValue &i_b, TReduceValue *o_reduceOutput),
		TReduceValue *o_reduceOutput
)
{
	CTraversalTask_GenericTreeNode_MidAndLeafNodes_Reduce<CLambdaFun1, CLambdaFun2, TReduceValue> t(this, i_lambda_leaves, i_lambda_midnodes, i_reduceOperator);
	t.execute();

	*o_reduceOutput = t.reduceValue;

	if (!this->isLeaf())
		i_lambda_midnodes(this, o_reduceOutput);
}



template <
	typename CLambdaFun1,
	typename CLambdaFun2,
	typename TReduceValue
>
void traverse_GenericTreeNode_MidAndLeafNodes_Reduce_Parallel(
		CLambdaFun1 i_lambda_leaves,
		CLambdaFun2 i_lambda_midnodes,
		void (*i_reduceOperator)(const TReduceValue &i_a, const TReduceValue &i_b, TReduceValue *o_reduceOutput),
		TReduceValue *o_reduceOutput
)
{
	CTraversalTask_GenericTreeNode_MidAndLeafNodes_Reduce<CLambdaFun1, CLambdaFun2, TReduceValue> t(this, i_lambda_leaves, i_lambda_midnodes, i_reduceOperator);
	t.execute();

	*o_reduceOutput = t.reduceValue;

	if (!this->isLeaf())
		i_lambda_midnodes(this, o_reduceOutput);
}



/*
 * serial/parallel
 */
template <
	bool parallelProcessing,
	typename CLambdaFun1,
	typename CLambdaFun2,
	typename TReduceValue
>
void traverse_GenericTreeNode_MidAndLeafNodes_Reduce(
		CLambdaFun1 i_lambda_leaves,
		CLambdaFun2 i_lambda_midnodes,
		void (*i_reduceOperator)(const TReduceValue &i_a, const TReduceValue &i_b, TReduceValue *o_reduceOutput),
		TReduceValue *o_reduceOutput
)
{
	if (parallelProcessing)
		traverse_GenericTreeNode_MidAndLeafNodes_Reduce_Parallel<CLambdaFun1, CLambdaFun2, TReduceValue>(i_lambda_leaves, i_lambda_midnodes, i_reduceOperator, o_reduceOutput);
	else
		traverse_GenericTreeNode_MidAndLeafNodes_Reduce_Serial<CLambdaFun1, CLambdaFun2, TReduceValue>(i_lambda_leaves, i_lambda_midnodes, i_reduceOperator, o_reduceOutput);
}

#endif
