/*
 * Copyright (C) 2011 Technische Universitaet Muenchen
 * This file is part of the Sierpi project. For conditions of distribution and
 * use, please see the copyright notice in the file 'copyright.txt' at the root
 * directory of this package and the copyright notice at http://www5.in.tum.de/sierpi
 *
 *
 *  Created on: Jan 10, 2011
 *      Author: Martin Schreiber (martin.schreiber@in.tum.de)
 */

#ifndef CTRAVERSATOR_VERTEXCOORDS_NORMALS_ELEMENTDATA_PARALLEL_HPP_
#define CTRAVERSATOR_VERTEXCOORDS_NORMALS_ELEMENTDATA_PARALLEL_HPP_

#include "libsierpi/stacks/CFBStacks.hpp"
#include "libsierpi/triangle/CTriangle_Factory.hpp"

namespace sierpi
{
namespace travs
{

template <class CKernelClass>
class CTraversator_VertexCoords_Normals_ElementData
{
public:
	/**
	 * implementation of kernel class
	 */
	CKernelClass cKernelClass;

	/**
	 * a few typedefs for convenience
	 */
	typedef typename CKernelClass::TElementData TElementData;
	typedef typename CKernelClass::TVertexScalar TVertexScalar;
	typedef CTraversator_VertexCoords_Normals_ElementData<CKernelClass> TThisClass;

	/**
	 * stack and lifo handlers used in this traversator
	 */
	CStackReaderTopDown<char> structure_lifo_in;
	CStackReaderTopDown<TElementData> element_data_lifo;


	/**
	 * include automagically generated code
	 */
#define SIERPI_PARALLEL_CODE	1
#include "auto/CTraversator_VertexCoords_Normals_ElementData_autogenerated.hpp"
#undef SIERPI_PARALLEL_CODE
#include "../CTraversatorClassInc_SetupSinglePass.hpp"


	void action(	CFBStacks<char> &p_structure_stacks,
					CFBStacks<TElementData> &p_element_data_stacks
	)
	{
		assert(p_structure_stacks.direction == p_element_data_stacks.direction);
		assert (p_structure_stacks.direction == CFBStacks<char>::FORWARD);

		structure_lifo_in.setup(p_structure_stacks.forward);
		element_data_lifo.setup(p_element_data_stacks.forward);

		cKernelClass.traversal_pre_hook();

#if CONFIG_TRAVERSATORS_WITH_NORMALS_AS_INTS
		(this->*sfcRecursiveMethod_Forward)(
				triangleFactory.vertices[0][0], triangleFactory.vertices[0][1],
				triangleFactory.vertices[1][0], triangleFactory.vertices[1][1],
				triangleFactory.vertices[2][0], triangleFactory.vertices[2][1],
				triangleFactory.hypNormal
			);
#else
		(this->*sfcRecursiveMethod_Forward)(
				triangleFactory.vertices[0][0], triangleFactory.vertices[0][1],
				triangleFactory.vertices[1][0], triangleFactory.vertices[1][1],
				triangleFactory.vertices[2][0], triangleFactory.vertices[2][1]
			);
#endif

		cKernelClass.traversal_post_hook();
	}
};

}
}

#endif /* CSTRUCTURE_VERTEXDATA_H_ */
