/*
 * CMainThreadingOMP.cpp
 *
 *  Created on: May 8, 2012
 *      Author: schreibm
 */

#include "CMainThreadingOMP.hpp"


#include <sys/types.h>
#include <unistd.h>
#include <linux/unistd.h>
#include <errno.h>
#include <pthread.h>
#include <assert.h>
#include <signal.h>
#include <cassert>

#include <sstream>
#include <ostream>

#include "CAffinityMap.hpp"

pid_t gettid()
{
	return (pid_t)syscall(__NR_gettid);
}



bool CMainThreadingOMP_setAffinities(
	int i_num_threads,
	int i_max_cores,
	int i_mode,
	int i_verbose_level
)
{
	assert(i_num_threads < 1024);

	int affinityMap[1024];
	if (!CAffinityMap::setup(i_num_threads, i_max_cores, i_mode, i_verbose_level, affinityMap))
		return true;

#pragma omp parallel for shared(affinityMap)  schedule(static,1)
	for (int i = 0; i < i_num_threads; i++)
	{
		cpu_set_t cpu_set;
		CPU_ZERO(&cpu_set);
		CPU_SET(affinityMap[i], &cpu_set);

		int retval = sched_setaffinity(gettid(), sizeof(cpu_set_t), &cpu_set);
		assert(retval == 0);
	}

	return true;
}
