/*
 * CMainThreadingTBB.cpp
 *
 *  Created on: May 8, 2012
 *      Author: schreibm
 */

#include "CMainThreadingTBB.hpp"

#include <tbb/atomic.h>
#include <tbb/task.h>

#include <sys/types.h>
#include <unistd.h>
#include <linux/unistd.h>
#include <errno.h>
#include <pthread.h>
#include <assert.h>
#include <signal.h>
#include <cassert>

#include <sstream>
#include <ostream>

#include "CAffinityMap.hpp"

/*
 * pieces of the following codes are from TBB 4.0, src/perf.cpp
 * http://threadingbuildingblocks.org/
 *
 * the code was slightly modified.
 */

pid_t gettid()
{
	return (pid_t)syscall(__NR_gettid);
}

bool PinTheThread(
		int cpu_idx,
		tbb::atomic<int>& nRemainingThreadsForPinning
)
{
	cpu_set_t orig_mask, target_mask;
	CPU_ZERO(&target_mask);
	CPU_SET(cpu_idx, &target_mask);
	assert(CPU_ISSET(cpu_idx, &target_mask));
	CPU_ZERO(&orig_mask);

	int res = sched_getaffinity(gettid(), sizeof(cpu_set_t), &orig_mask);
	assert(res == 0);

	res = sched_setaffinity(gettid(), sizeof(cpu_set_t), &target_mask);
	assert(res == 0);

	--nRemainingThreadsForPinning;
	while (nRemainingThreadsForPinning)
		__TBB_Yield();

	return true;
}


class AffinitySetterTask :
	public tbb::task
{
public:
	static bool m_result;
	static tbb::atomic<int> m_nThreads;
	int m_idx;

	tbb::task* execute ()
	{
		m_result = PinTheThread(m_idx, m_nThreads);
		return NULL;
	}

public:
	AffinitySetterTask(int idx) :
		m_idx(idx)
	{}

	friend bool AffinitizeTBB ( int, int /*mode*/ );
};


bool AffinitySetterTask::m_result;
tbb::atomic<int> AffinitySetterTask::m_nThreads;



bool CMainThreadingTBB_setAffinities(
		int i_num_threads,
		int i_max_cores,
		int i_mode,
		int i_verbose_level
)
{
	AffinitySetterTask::m_result = true;
	AffinitySetterTask::m_nThreads = i_num_threads;

	assert(i_num_threads < 1024);
	int affinityMap[1024];

	if (!CAffinityMap::setup(i_num_threads, i_max_cores, i_mode, i_verbose_level, affinityMap))
		return true;

	tbb::task_list  tl;
	for (int i = 0; i < i_num_threads; i++)
	{
		tbb::task &t = *new(tbb::task::allocate_root()) AffinitySetterTask(affinityMap[i]);
		t.set_affinity(tbb::task::affinity_id(i+1));
		tl.push_back(t);
	}
	tbb::task::spawn_root_and_wait(tl);

	return AffinitySetterTask::m_result;
}
