/*
 * Copyright (C) 2011 Technische Universitaet Muenchen
 * This file is part of the Sierpi project. For conditions of distribution and
 * use, please see the copyright notice in the file 'copyright.txt' at the root
 * directory of this package and the copyright notice at http://www5.in.tum.de/sierpi
 *
 *  Created on: Feb 2, 2011
 *      Author: Martin Schreiber (martin.schreiber@in.tum.de)
 */

#ifndef COUTPUTVTK_VERTICES_ELEMENT_TSUNAMI_HPP_
#define COUTPUTVTK_VERTICES_ELEMENT_TSUNAMI_HPP_

#include "libsierpi/traversators/vertexCoords/CTraversator_VertexCoords_ElementData.hpp"
#include "simulations/tsunami_common/types/CTsunamiTypes.hpp"
#include "libmath/CVertex2d.hpp"

namespace sierpi
{
namespace kernels
{


#define VISUALIZATION_SIMPLE						1

// TODO: code has to be fixed with bathymetry for this backend
#define VISUALIZATION_TRIANGLES_AT_EDGE_MIDPOINTS	0

#define VISUALIZATION_ALIGNED_TRIANGLES				0


template <typename p_CSimulationStacks, int i_visualizationType>
class COutputVTK_Vertices_Element_Tsunami
{
public:
	typedef typename p_CSimulationStacks::TElementData	TElementData;

	typedef float TVertexScalar;
	typedef CVertex2d<TVertexScalar> TVertexType;

	typedef sierpi::travs::CTraversator_VertexCoords_ElementData<COutputVTK_Vertices_Element_Tsunami<p_CSimulationStacks, i_visualizationType>, p_CSimulationStacks > TRAV;

	std::ofstream *vtk_file_output_stream;

private:
	TVertexScalar *last_triangle;
	size_t max_triangles;

public:
	inline void elementAction(
			TVertexScalar vx1, TVertexScalar vy1,
			TVertexScalar vx2, TVertexScalar vy2,
			TVertexScalar vx3, TVertexScalar vy3,
			TElementData *i_ElementData
	)
	{
#define fixHeight(h)	(h)

#if VISUALIZATION_SIMPLE

	if (i_visualizationType == 0)
	{
		(*vtk_file_output_stream)
				<< vx1 << " "
				<< vy1 << " "
				<< 0 << std::endl;

		(*vtk_file_output_stream)
				<< vx2 << " "
				<< vy2 << " "
				<< 0 << std::endl;

		(*vtk_file_output_stream)
				<< vx3 << " "
				<< vy3 << " "
				<< 0 << std::endl;
	}
	else if (i_visualizationType == 1)
	{
		/*
		 * SURFACE coordinates
		 */
#if SIMULATION_TSUNAMI_ORDER_OF_BASIS_FUNCTIONS==0
		TVertexScalar h = i_ElementData->dofs.h;
#else
		TVertexScalar h = (1.0/3.0)*(i_ElementData->hyp_edge.b + i_ElementData->left_edge.b + i_ElementData->right_edge.b + i_ElementData->hyp_edge.h + i_ElementData->left_edge.h + i_ElementData->right_edge.h);
#endif

		(*vtk_file_output_stream) << h << std::endl;
	}
	else if (i_visualizationType == 2)
	{
	/*
	 * MOMENTUM HU
	 */
#if SIMULATION_TSUNAMI_ORDER_OF_BASIS_FUNCTIONS==0
		TVertexScalar hu = i_ElementData->dofs.qx;
#else
		TVertexScalar hu = (1.0/3.0)*(i_ElementData->hyp_edge.qx + i_ElementData->left_edge.qx + i_ElementData->right_edge.qx);
#endif

		(*vtk_file_output_stream) << hu << std::endl;
	}
	else if (i_visualizationType == 3)
	{
	/*
	 * MOMENTUM HV
	 */
#if SIMULATION_TSUNAMI_ORDER_OF_BASIS_FUNCTIONS==0
		TVertexScalar hv = i_ElementData->dofs.qy;
#else
		TVertexScalar hv = (1.0/3.0)*(i_ElementData->hyp_edge.qy + i_ElementData->left_edge.qy + i_ElementData->right_edge.qy);
#endif

		(*vtk_file_output_stream) << hv << std::endl;
	}
	else if (i_visualizationType == 4)
	{
	/*
	 * BATHYMETRY
	 */
#if SIMULATION_TSUNAMI_ORDER_OF_BASIS_FUNCTIONS==0
		TVertexScalar b = i_ElementData->dofs.b;
#else
		TVertexScalar b = (1.0/3.0)*(i_ElementData->hyp_edge.b + i_ElementData->left_edge.b + i_ElementData->right_edge.b);
#endif

		(*vtk_file_output_stream) << b << std::endl;
	}


#endif

#if VISUALIZATION_TRIANGLES_AT_EDGE_MIDPOINTS

		TVertexScalar hyp_x = (vx1+vx2)*0.5;
		TVertexScalar hyp_y = (vy1+vy2)*0.5;

		TVertexScalar right_edge_x = (vx2+vx3)*0.5;
		TVertexScalar right_edge_y = (vy2+vy3)*0.5;

		TVertexScalar left_edge_x = (vx3+vx1)*0.5;
		TVertexScalar left_edge_y = (vy3+vy1)*0.5;

		(*vtk_file_output_stream)
				<<	hyp_x << " "
				<< i_ElementData->hyp_edge.h << " "
				<< -hyp_y << std::endl;

		(*vtk_file_output_stream)
				<< left_edge_x << " "
				<< i_ElementData->left_edge.h << " "
				<< -left_edge_y << std::endl;

		(*vtk_file_output_stream)
				<< right_edge_x << " "
				<< i_ElementData->right_edge.h << " "
				<< -right_edge_y << std::endl;
#endif

#if VISUALIZATION_ALIGNED_TRIANGLES
		TVertexScalar hyp_edge_x = (vx1+vx2)*0.5;
		TVertexScalar hyp_edge_y = (vy1+vy2)*0.5;

		TVertexScalar right_edge_x = (vx2+vx3)*0.5;
		TVertexScalar right_edge_y = (vy2+vy3)*0.5;

		TVertexScalar left_edge_x = (vx3+vx1)*0.5;
		TVertexScalar left_edge_y = (vy3+vy1)*0.5;

		TVertexScalar hyp_dx = right_edge_x - left_edge_x;
		TVertexScalar hyp_dy = right_edge_y - left_edge_y;
		TVertexScalar hyp_dh = i_ElementData->right_edge.h - i_ElementData->left_edge.h;

		TVertexScalar left_dx = right_edge_x - hyp_edge_x;
		TVertexScalar left_dy = right_edge_y - hyp_edge_y;
		TVertexScalar left_dh = i_ElementData->right_edge.h - i_ElementData->hyp_edge.h;

		(*vtk_file_output_stream)
					<< (hyp_edge_x-hyp_dx) << " "
					<< (i_ElementData->hyp_edge.h-hyp_dh) << " "
					<< -(hyp_edge_y-hyp_dy) << std::endl;

		(*vtk_file_output_stream)
					<< (hyp_edge_x+hyp_dx) << " "
					<< (i_ElementData->hyp_edge.h+hyp_dh) << " "
					<< -(hyp_edge_y+hyp_dy) << std::endl;

		(*vtk_file_output_stream)
					<< (left_edge_x+left_dx) << " "
					<< (i_ElementData->left_edge.h+left_dh) << " "
					<< -(left_edge_y+left_dy) << std::endl;
#endif
	}
#undef fixHeight

	inline void traversal_pre_hook()
	{
	}

	inline void traversal_post_hook()
	{
	}

	inline void setup(std::ofstream *p_vtk_file_output_stream)
	{
		vtk_file_output_stream = p_vtk_file_output_stream;
	}

	/**
	 * setup traversator based on parent triangle
	 */
	void setup_WithKernel(
			COutputVTK_Vertices_Element_Tsunami &parent_kernel
	)
	{
	}


	inline void setup_WithParameters(
			std::ofstream *p_vtk_file_output_stream
		)
	{
	}
};


#undef VISUALIZATION_SIMPLE
#undef VISUALIZATION_TRIANGLES_AT_EDGE_MIDPOINTS
#undef VISUALIZATION_ALIGNED_TRIANGLES

}
}

#endif
